@@ -1,25 +1,13 @@
-# -*- coding: utf-8 -*-
-#    Copyright (C) 2004-2018 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
-#
-# Authors: Aric Hagberg (hagberg@lanl.gov)
-#          Pieter Swart (swart@lanl.gov)
-#          Sasha Gutfraind (ag362@cornell.edu)
 """Load centrality."""
-from __future__ import division
 from operator import itemgetter
 
 import networkx as nx
 
-__all__ = ['load_centrality', 'edge_load_centrality']
+__all__ = ["load_centrality", "edge_load_centrality"]
 
 
-def newman_betweenness_centrality(G, v=None, cutoff=None,
-                                  normalized=True, weight=None):
+@nx._dispatch(edge_attrs="weight")
+def newman_betweenness_centrality(G, v=None, cutoff=None, normalized=True, weight=None):
     """Compute load centrality for nodes.
 
     The load centrality of a node is the fraction of all shortest
@@ -37,6 +25,7 @@ def newman_betweenness_centrality(G, v=None, cutoff=None,
     weight : None or string, optional (default=None)
       If None, edge weights are ignored.
       Otherwise holds the name of the edge attribute used as weight.
+      The weight of an edge is treated as the length or distance between the two sides.
 
     cutoff : bool, optional (default=None)
       If specified, only consider paths of length <= cutoff.
@@ -48,7 +37,7 @@ def newman_betweenness_centrality(G, v=None, cutoff=None,
 
     See Also
     --------
-    betweenness_centrality()
+    betweenness_centrality
 
     Notes
     -----
@@ -65,9 +54,9 @@ def newman_betweenness_centrality(G, v=None, cutoff=None,
     .. [2] Kwang-Il Goh, Byungnam Kahng and Doochul Kim
        Universal behavior of Load Distribution in Scale-Free Networks.
        Physical Review Letters 87(27):1–4, 2001.
-       http://phya.snu.ac.kr/~dkim/PRL87278701.pdf
+       https://doi.org/10.1103/PhysRevLett.87.278701
     """
-    if v is not None:   # only one node
+    if v is not None:  # only one node
         betweenness = 0.0
         for source in G:
             ubetween = _node_betweenness(G, source, cutoff, False, weight)
@@ -77,7 +66,6 @@ def newman_betweenness_centrality(G, v=None, cutoff=None,
             if order <= 2:
                 return betweenness  # no normalization b=0 for all nodes
             betweenness *= 1.0 / ((order - 1) * (order - 2))
-        return betweenness
     else:
         betweenness = {}.fromkeys(G, 0.0)
         for source in betweenness:
@@ -91,11 +79,10 @@ def newman_betweenness_centrality(G, v=None, cutoff=None,
             scale = 1.0 / ((order - 1) * (order - 2))
             for v in betweenness:
                 betweenness[v] *= scale
-        return betweenness  # all nodes
+    return betweenness  # all nodes
 
 
-def _node_betweenness(G, source, cutoff=False, normalized=True,
-                      weight=None):
+def _node_betweenness(G, source, cutoff=False, normalized=True, weight=None):
     """Node betweenness_centrality helper:
 
     See betweenness_centrality for what you probably want.
@@ -112,11 +99,9 @@ def _node_betweenness(G, source, cutoff=False, normalized=True,
     """
     # get the predecessor and path length data
     if weight is None:
-        (pred, length) = nx.predecessor(G, source, cutoff=cutoff,
-                                        return_seen=True)
+        (pred, length) = nx.predecessor(G, source, cutoff=cutoff, return_seen=True)
     else:
-        (pred, length) = nx.dijkstra_predecessor_and_distance(G, source,
-                                                              cutoff, weight)
+        (pred, length) = nx.dijkstra_predecessor_and_distance(G, source, cutoff, weight)
 
     # order the nodes by path length
     onodes = [(l, vert) for (vert, l) in length.items()]
@@ -130,10 +115,10 @@ def _node_betweenness(G, source, cutoff=False, normalized=True,
         v = onodes.pop()
         if v in pred:
             num_paths = len(pred[v])  # Discount betweenness if more than
-            for x in pred[v]:         # one shortest path.
+            for x in pred[v]:  # one shortest path.
                 if x == source:  # stop if hit source because all remaining v
-                    break        # also have pred[v]==[source]
-                between[x] += between[v] / float(num_paths)
+                    break  # also have pred[v]==[source]
+                between[x] += between[v] / num_paths
     #  remove source
     for v in between:
         between[v] -= 1
@@ -142,7 +127,7 @@ def _node_betweenness(G, source, cutoff=False, normalized=True,
         l = len(between)
         if l > 2:
             # scale by 1/the number of possible paths
-            scale = 1.0 / float((l - 1) * (l - 2))
+            scale = 1 / ((l - 1) * (l - 2))
             for v in between:
                 between[v] *= scale
     return between
@@ -151,6 +136,7 @@ def _node_betweenness(G, source, cutoff=False, normalized=True,
 load_centrality = newman_betweenness_centrality
 
 
+@nx._dispatch
 def edge_load_centrality(G, cutoff=False):
     """Compute edge load.
 
@@ -198,7 +184,7 @@ def _edge_betweenness(G, source, nodes=None, cutoff=False):
         between[(u, v)] = 1.0
         between[(v, u)] = 1.0
 
-    while onodes:           # work through all paths
+    while onodes:  # work through all paths
         v = onodes.pop()
         if v in pred:
             # Discount betweenness if more than one shortest path.
