@@ -1,24 +1,14 @@
-# -*- coding: utf-8 -*-
 """
 Minimum cost flow algorithms on directed connected graphs.
 """
 
-__author__ = """Loïc Séguin-C. <loicseguin@gmail.com>"""
-# Copyright (C) 2010 Loïc Séguin-C. <loicseguin@gmail.com>
-# All rights reserved.
-# BSD license.
-
-
-__all__ = ['min_cost_flow_cost',
-           'min_cost_flow',
-           'cost_of_flow',
-           'max_flow_min_cost']
+__all__ = ["min_cost_flow_cost", "min_cost_flow", "cost_of_flow", "max_flow_min_cost"]
 
 import networkx as nx
 
 
-def min_cost_flow_cost(G, demand = 'demand', capacity = 'capacity',
-                        weight = 'weight'):
+@nx._dispatch(node_attrs="demand", edge_attrs={"capacity": float("inf"), "weight": 0})
+def min_cost_flow_cost(G, demand="demand", capacity="capacity", weight="weight"):
     r"""Find the cost of a minimum cost flow satisfying all demands in digraph G.
 
     G is a digraph with edge costs and capacities and in which nodes
@@ -93,25 +83,23 @@ def min_cost_flow_cost(G, demand = 'demand', capacity = 'capacity',
     --------
     A simple example of a min cost flow problem.
 
-    >>> import networkx as nx
     >>> G = nx.DiGraph()
-    >>> G.add_node('a', demand = -5)
-    >>> G.add_node('d', demand = 5)
-    >>> G.add_edge('a', 'b', weight = 3, capacity = 4)
-    >>> G.add_edge('a', 'c', weight = 6, capacity = 10)
-    >>> G.add_edge('b', 'd', weight = 1, capacity = 9)
-    >>> G.add_edge('c', 'd', weight = 2, capacity = 5)
+    >>> G.add_node("a", demand=-5)
+    >>> G.add_node("d", demand=5)
+    >>> G.add_edge("a", "b", weight=3, capacity=4)
+    >>> G.add_edge("a", "c", weight=6, capacity=10)
+    >>> G.add_edge("b", "d", weight=1, capacity=9)
+    >>> G.add_edge("c", "d", weight=2, capacity=5)
     >>> flowCost = nx.min_cost_flow_cost(G)
     >>> flowCost
     24
     """
-    return nx.network_simplex(G, demand = demand, capacity = capacity,
-                              weight = weight)[0]
+    return nx.network_simplex(G, demand=demand, capacity=capacity, weight=weight)[0]
 
 
-def min_cost_flow(G, demand = 'demand', capacity = 'capacity',
-                  weight = 'weight'):
-    r"""Return a minimum cost flow satisfying all demands in digraph G.
+@nx._dispatch(node_attrs="demand", edge_attrs={"capacity": float("inf"), "weight": 0})
+def min_cost_flow(G, demand="demand", capacity="capacity", weight="weight"):
+    r"""Returns a minimum cost flow satisfying all demands in digraph G.
 
     G is a digraph with edge costs and capacities and in which nodes
     have demand, i.e., they want to send or receive some amount of
@@ -186,21 +174,20 @@ def min_cost_flow(G, demand = 'demand', capacity = 'capacity',
     --------
     A simple example of a min cost flow problem.
 
-    >>> import networkx as nx
     >>> G = nx.DiGraph()
-    >>> G.add_node('a', demand = -5)
-    >>> G.add_node('d', demand = 5)
-    >>> G.add_edge('a', 'b', weight = 3, capacity = 4)
-    >>> G.add_edge('a', 'c', weight = 6, capacity = 10)
-    >>> G.add_edge('b', 'd', weight = 1, capacity = 9)
-    >>> G.add_edge('c', 'd', weight = 2, capacity = 5)
+    >>> G.add_node("a", demand=-5)
+    >>> G.add_node("d", demand=5)
+    >>> G.add_edge("a", "b", weight=3, capacity=4)
+    >>> G.add_edge("a", "c", weight=6, capacity=10)
+    >>> G.add_edge("b", "d", weight=1, capacity=9)
+    >>> G.add_edge("c", "d", weight=2, capacity=5)
     >>> flowDict = nx.min_cost_flow(G)
     """
-    return nx.network_simplex(G, demand = demand, capacity = capacity,
-                              weight = weight)[1]
+    return nx.network_simplex(G, demand=demand, capacity=capacity, weight=weight)[1]
 
 
-def cost_of_flow(G, flowDict, weight = 'weight'):
+@nx._dispatch(edge_attrs={"weight": 0})
+def cost_of_flow(G, flowDict, weight="weight"):
     """Compute the cost of the flow given by flowDict on graph G.
 
     Note that this function does not check for the validity of the
@@ -241,12 +228,12 @@ def cost_of_flow(G, flowDict, weight = 'weight'):
     multiplying the relevant edge attributes by a convenient
     constant factor (eg 100).
     """
-    return sum((flowDict[u][v] * d.get(weight, 0)
-                for u, v, d in G.edges(data = True)))
+    return sum((flowDict[u][v] * d.get(weight, 0) for u, v, d in G.edges(data=True)))
 
 
-def max_flow_min_cost(G, s, t, capacity = 'capacity', weight = 'weight'):
-    """Return a maximum (s, t)-flow of minimum cost.
+@nx._dispatch(edge_attrs={"capacity": float("inf"), "weight": 0})
+def max_flow_min_cost(G, s, t, capacity="capacity", weight="weight"):
+    """Returns a maximum (s, t)-flow of minimum cost.
 
     G is a digraph with edge costs and capacities. There is a source
     node s and a sink node t. This function finds a maximum flow from
@@ -310,18 +297,22 @@ def max_flow_min_cost(G, s, t, capacity = 'capacity', weight = 'weight'):
     Examples
     --------
     >>> G = nx.DiGraph()
-    >>> G.add_edges_from([(1, 2, {'capacity': 12, 'weight': 4}),
-    ...                   (1, 3, {'capacity': 20, 'weight': 6}),
-    ...                   (2, 3, {'capacity': 6, 'weight': -3}),
-    ...                   (2, 6, {'capacity': 14, 'weight': 1}),
-    ...                   (3, 4, {'weight': 9}),
-    ...                   (3, 5, {'capacity': 10, 'weight': 5}),
-    ...                   (4, 2, {'capacity': 19, 'weight': 13}),
-    ...                   (4, 5, {'capacity': 4, 'weight': 0}),
-    ...                   (5, 7, {'capacity': 28, 'weight': 2}),
-    ...                   (6, 5, {'capacity': 11, 'weight': 1}),
-    ...                   (6, 7, {'weight': 8}),
-    ...                   (7, 4, {'capacity': 6, 'weight': 6})])
+    >>> G.add_edges_from(
+    ...     [
+    ...         (1, 2, {"capacity": 12, "weight": 4}),
+    ...         (1, 3, {"capacity": 20, "weight": 6}),
+    ...         (2, 3, {"capacity": 6, "weight": -3}),
+    ...         (2, 6, {"capacity": 14, "weight": 1}),
+    ...         (3, 4, {"weight": 9}),
+    ...         (3, 5, {"capacity": 10, "weight": 5}),
+    ...         (4, 2, {"capacity": 19, "weight": 13}),
+    ...         (4, 5, {"capacity": 4, "weight": 0}),
+    ...         (5, 7, {"capacity": 28, "weight": 2}),
+    ...         (6, 5, {"capacity": 11, "weight": 1}),
+    ...         (6, 7, {"weight": 8}),
+    ...         (7, 4, {"capacity": 6, "weight": 6}),
+    ...     ]
+    ... )
     >>> mincostFlow = nx.max_flow_min_cost(G, 1, 7)
     >>> mincost = nx.cost_of_flow(G, mincostFlow)
     >>> mincost
@@ -330,14 +321,15 @@ def max_flow_min_cost(G, s, t, capacity = 'capacity', weight = 'weight'):
     >>> maxFlow = maximum_flow(G, 1, 7)[1]
     >>> nx.cost_of_flow(G, maxFlow) >= mincost
     True
-    >>> mincostFlowValue = (sum((mincostFlow[u][7] for u in G.predecessors(7)))
-    ...                     - sum((mincostFlow[7][v] for v in G.successors(7))))
+    >>> mincostFlowValue = sum((mincostFlow[u][7] for u in G.predecessors(7))) - sum(
+    ...     (mincostFlow[7][v] for v in G.successors(7))
+    ... )
     >>> mincostFlowValue == nx.maximum_flow_value(G, 1, 7)
     True
 
     """
-    maxFlow = nx.maximum_flow_value(G, s, t, capacity = capacity)
+    maxFlow = nx.maximum_flow_value(G, s, t, capacity=capacity)
     H = nx.DiGraph(G)
-    H.add_node(s, demand = -maxFlow)
-    H.add_node(t, demand = maxFlow)
-    return min_cost_flow(H, capacity = capacity, weight = weight)
+    H.add_node(s, demand=-maxFlow)
+    H.add_node(t, demand=maxFlow)
+    return min_cost_flow(H, capacity=capacity, weight=weight)
