@@ -4,6 +4,7 @@ import networkx as nx
 __all__ = ["complement", "reverse"]
 
 
+@nx._dispatch
 def complement(G):
     """Returns the graph complement of G.
 
@@ -17,11 +18,19 @@ def complement(G):
     GC : A new graph.
 
     Notes
-    ------
-    Note that complement() does not create self-loops and also
+    -----
+    Note that `complement` does not create self-loops and also
     does not produce parallel edges for MultiGraphs.
 
     Graph, node, and edge data are not propagated to the new graph.
+
+    Examples
+    --------
+    >>> G = nx.Graph([(1, 2), (1, 3), (2, 3), (3, 4), (3, 5)])
+    >>> G_complement = nx.complement(G)
+    >>> G_complement.edges() # This shows the edges of the complemented graph
+    EdgeView([(1, 4), (1, 5), (2, 4), (2, 5), (4, 5)])
+
     """
     R = G.__class__()
     R.add_nodes_from(G)
@@ -31,6 +40,7 @@ def complement(G):
     return R
 
 
+@nx._dispatch
 def reverse(G, copy=True):
     """Returns the reverse directed graph of G.
 
@@ -47,6 +57,18 @@ def reverse(G, copy=True):
     H : directed graph
         The reversed G.
 
+    Raises
+    ------
+    NetworkXError
+        If graph is undirected.
+
+    Examples
+    --------
+    >>> G = nx.DiGraph([(1, 2), (1, 3), (2, 3), (3, 4), (3, 5)])
+    >>> G_reversed = nx.reverse(G)
+    >>> G_reversed.edges()
+    OutEdgeView([(2, 1), (3, 1), (3, 2), (4, 3), (5, 3)])
+
     """
     if not G.is_directed():
         raise nx.NetworkXError("Cannot reverse an undirected graph.")
