@@ -1,49 +1,60 @@
-#    Copyright (C) 2004-2018 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
-#
-# Authors: Aric Hagberg <hagberg@lanl.gov>
-#          Pieter Swart <swart@lanl.gov>
-#          Dan Schult <dschult@colgate.edu>
 """Functional interface to graph methods and assorted utilities.
 """
-from __future__ import division
 
 from collections import Counter
 from itertools import chain
-try:
-    from itertools import zip_longest
-except ImportError:
-    from itertools import izip_longest as zip_longest
 
 import networkx as nx
-from networkx.utils import pairwise, not_implemented_for
+from networkx.utils import not_implemented_for, pairwise
 
-__all__ = ['nodes', 'edges', 'degree', 'degree_histogram', 'neighbors',
-           'number_of_nodes', 'number_of_edges', 'density',
-           'is_directed', 'info', 'freeze', 'is_frozen', 'subgraph',
-           'induced_subgraph', 'edge_subgraph', 'restricted_view',
-           'reverse_view', 'to_directed', 'to_undirected',
-           'add_star', 'add_path', 'add_cycle',
-           'create_empty_copy', 'set_node_attributes',
-           'get_node_attributes', 'set_edge_attributes',
-           'get_edge_attributes', 'all_neighbors', 'non_neighbors',
-           'non_edges', 'common_neighbors', 'is_weighted',
-           'is_negatively_weighted', 'is_empty',
-           'selfloop_edges', 'nodes_with_selfloops', 'number_of_selfloops',
-           ]
+__all__ = [
+    "nodes",
+    "edges",
+    "degree",
+    "degree_histogram",
+    "neighbors",
+    "number_of_nodes",
+    "number_of_edges",
+    "density",
+    "is_directed",
+    "freeze",
+    "is_frozen",
+    "subgraph",
+    "induced_subgraph",
+    "edge_subgraph",
+    "restricted_view",
+    "to_directed",
+    "to_undirected",
+    "add_star",
+    "add_path",
+    "add_cycle",
+    "create_empty_copy",
+    "set_node_attributes",
+    "get_node_attributes",
+    "set_edge_attributes",
+    "get_edge_attributes",
+    "all_neighbors",
+    "non_neighbors",
+    "non_edges",
+    "common_neighbors",
+    "is_weighted",
+    "is_negatively_weighted",
+    "is_empty",
+    "selfloop_edges",
+    "nodes_with_selfloops",
+    "number_of_selfloops",
+    "path_weight",
+    "is_path",
+]
 
 
 def nodes(G):
-    """Return an iterator over the graph nodes."""
+    """Returns an iterator over the graph nodes."""
     return G.nodes()
 
 
 def edges(G, nbunch=None):
-    """Return an edge view of edges incident to nodes in nbunch.
+    """Returns an edge view of edges incident to nodes in nbunch.
 
     Return all edges if nbunch is unspecified or nbunch=None.
 
@@ -53,29 +64,29 @@ def edges(G, nbunch=None):
 
 
 def degree(G, nbunch=None, weight=None):
-    """Return a degree view of single node or of nbunch of nodes.
+    """Returns a degree view of single node or of nbunch of nodes.
     If nbunch is omitted, then return degrees of *all* nodes.
     """
     return G.degree(nbunch, weight)
 
 
 def neighbors(G, n):
-    """Return a list of nodes connected to node n. """
+    """Returns a list of nodes connected to node n."""
     return G.neighbors(n)
 
 
 def number_of_nodes(G):
-    """Return the number of nodes in the graph."""
+    """Returns the number of nodes in the graph."""
     return G.number_of_nodes()
 
 
 def number_of_edges(G):
-    """Return the number of edges in the graph. """
+    """Returns the number of edges in the graph."""
     return G.number_of_edges()
 
 
 def density(G):
-    r"""Return the density of a graph.
+    r"""Returns the density of a graph.
 
     The density for undirected graphs is
 
@@ -110,7 +121,7 @@ def density(G):
 
 
 def degree_histogram(G):
-    """Return a list of the frequency of each degree value.
+    """Returns a list of the frequency of each degree value.
 
     Parameters
     ----------
@@ -133,11 +144,11 @@ def degree_histogram(G):
 
 
 def is_directed(G):
-    """ Return True if graph is directed."""
+    """Return True if graph is directed."""
     return G.is_directed()
 
 
-def frozen(*args):
+def frozen(*args, **kwargs):
     """Dummy method for raising errors when trying to modify frozen graphs"""
     raise nx.NetworkXError("Frozen graph can't be modified")
 
@@ -158,9 +169,9 @@ def freeze(G):
     >>> G = nx.path_graph(4)
     >>> G = nx.freeze(G)
     >>> try:
-    ...    G.add_edge(4, 5)
-    ... except nx.NetworkXError as e:
-    ...    print(str(e))
+    ...     G.add_edge(4, 5)
+    ... except nx.NetworkXError as err:
+    ...     print(str(err))
     Frozen graph can't be modified
 
     Notes
@@ -187,12 +198,13 @@ def freeze(G):
     G.remove_edge = frozen
     G.remove_edges_from = frozen
     G.clear = frozen
+    G.clear_edges = frozen
     G.frozen = True
     return G
 
 
 def is_frozen(G):
-    """Return True if graph is frozen.
+    """Returns True if graph is frozen.
 
     Parameters
     ----------
@@ -235,7 +247,11 @@ def add_star(G_to_add_to, nodes_for_star, **attr):
     >>> nx.add_star(G, [10, 11, 12], weight=2)
     """
     nlist = iter(nodes_for_star)
-    v = next(nlist)
+    try:
+        v = next(nlist)
+    except StopIteration:
+        return
+    G_to_add_to.add_node(v)
     edges = ((v, n) for n in nlist)
     G_to_add_to.add_edges_from(edges, **attr)
 
@@ -291,15 +307,23 @@ def add_cycle(G_to_add_to, nodes_for_cycle, **attr):
 
     Examples
     --------
-    >>> G = nx.Graph()   # or DiGraph, MultiGraph, MultiDiGraph, etc
+    >>> G = nx.Graph()  # or DiGraph, MultiGraph, MultiDiGraph, etc
     >>> nx.add_cycle(G, [0, 1, 2, 3])
     >>> nx.add_cycle(G, [10, 11, 12], weight=7)
     """
-    G_to_add_to.add_edges_from(pairwise(nodes_for_cycle, cyclic=True), **attr)
+    nlist = iter(nodes_for_cycle)
+    try:
+        first_node = next(nlist)
+    except StopIteration:
+        return
+    G_to_add_to.add_node(first_node)
+    G_to_add_to.add_edges_from(
+        pairwise(chain((first_node,), nlist), cyclic=True), **attr
+    )
 
 
 def subgraph(G, nbunch):
-    """Return the subgraph induced on nodes in nbunch.
+    """Returns the subgraph induced on nodes in nbunch.
 
     Parameters
     ----------
@@ -322,7 +346,7 @@ def subgraph(G, nbunch):
 
 
 def induced_subgraph(G, nbunch):
-    """Return a SubGraph view of `G` showing only nodes in nbunch.
+    """Returns a SubGraph view of `G` showing only nodes in nbunch.
 
     The induced subgraph of a graph on a set of nodes N is the
     graph with nodes N and edges from G which have both ends in N.
@@ -356,14 +380,15 @@ def induced_subgraph(G, nbunch):
 
     Examples
     --------
-    >>> import networkx as nx
     >>> G = nx.path_graph(4)  # or DiGraph, MultiGraph, MultiDiGraph, etc
-    >>> H = G.subgraph([0, 1, 2])
+    >>> H = nx.induced_subgraph(G, [0, 1, 3])
     >>> list(H.edges)
-    [(0, 1), (1, 2)]
+    [(0, 1)]
+    >>> list(H.nodes)
+    [0, 1, 3]
     """
     induced_nodes = nx.filters.show_nodes(G.nbunch_iter(nbunch))
-    return nx.graphviews.subgraph_view(G, induced_nodes)
+    return nx.subgraph_view(G, filter_node=induced_nodes)
 
 
 def edge_subgraph(G, edges):
@@ -399,7 +424,6 @@ def edge_subgraph(G, edges):
 
     Examples
     --------
-    >>> import networkx as nx
     >>> G = nx.path_graph(5)
     >>> H = G.edge_subgraph([(0, 1), (3, 4)])
     >>> list(H.nodes)
@@ -423,7 +447,7 @@ def edge_subgraph(G, edges):
             induced_edges = nxf.show_diedges(edges)
         else:
             induced_edges = nxf.show_edges(edges)
-    return nx.graphviews.subgraph_view(G, induced_nodes, induced_edges)
+    return nx.subgraph_view(G, filter_node=induced_nodes, filter_edge=induced_edges)
 
 
 def restricted_view(G, nodes, edges):
@@ -460,7 +484,6 @@ def restricted_view(G, nodes, edges):
 
     Examples
     --------
-    >>> import networkx as nx
     >>> G = nx.path_graph(5)
     >>> H = nx.restricted_view(G, [0], [(1, 2), (3, 4)])
     >>> list(H.nodes)
@@ -480,30 +503,21 @@ def restricted_view(G, nodes, edges):
             hide_edges = nxf.hide_diedges(edges)
         else:
             hide_edges = nxf.hide_edges(edges)
-    return nx.graphviews.subgraph_view(G, hide_nodes, hide_edges)
-
-
-@not_implemented_for('undirected')
-def reverse_view(digraph):
-    """Provide a reverse view of the digraph with edges reversed.
-
-    Identical to digraph.reverse(copy=False)
-    """
-    return nx.graphviews.reverse_view(digraph)
+    return nx.subgraph_view(G, filter_node=hide_nodes, filter_edge=hide_edges)
 
 
 def to_directed(graph):
-    """Return a directed view of the graph `graph`.
+    """Returns a directed view of the graph `graph`.
 
     Identical to graph.to_directed(as_view=True)
     Note that graph.to_directed defaults to `as_view=False`
     while this function always provides a view.
     """
-    return graph.to_directed()
+    return graph.to_directed(as_view=True)
 
 
 def to_undirected(graph):
-    """Return an undirected view of the graph `graph`.
+    """Returns an undirected view of the graph `graph`.
 
     Identical to graph.to_undirected(as_view=True)
     Note that graph.to_undirected defaults to `as_view=False`
@@ -513,7 +527,7 @@ def to_undirected(graph):
 
 
 def create_empty_copy(G, with_data=True):
-    """Return a copy of the graph G with all of the edges removed.
+    """Returns a copy of the graph G with all of the edges removed.
 
     Parameters
     ----------
@@ -524,55 +538,17 @@ def create_empty_copy(G, with_data=True):
        Propagate Graph and Nodes data to the new graph.
 
     See Also
-    -----
+    --------
     empty_graph
 
     """
-    H = G.fresh_copy()
+    H = G.__class__()
     H.add_nodes_from(G.nodes(data=with_data))
     if with_data:
         H.graph.update(G.graph)
     return H
 
 
-def info(G, n=None):
-    """Print short summary of information for the graph G or the node n.
-
-    Parameters
-    ----------
-    G : Networkx graph
-       A graph
-    n : node (any hashable)
-       A node in the graph G
-    """
-    info = ''  # append this all to a string
-    if n is None:
-        info += "Name: %s\n" % G.name
-        type_name = [type(G).__name__]
-        info += "Type: %s\n" % ",".join(type_name)
-        info += "Number of nodes: %d\n" % G.number_of_nodes()
-        info += "Number of edges: %d\n" % G.number_of_edges()
-        nnodes = G.number_of_nodes()
-        if len(G) > 0:
-            if G.is_directed():
-                deg = sum(d for n, d in G.in_degree()) / float(nnodes)
-                info += "Average in degree: %8.4f\n" % deg
-                deg = sum(d for n, d in G.out_degree()) / float(nnodes)
-                info += "Average out degree: %8.4f" % deg
-            else:
-                s = sum(dict(G.degree()).values())
-                info += "Average degree: %8.4f" % (float(s) / float(nnodes))
-
-    else:
-        if n not in G:
-            raise nx.NetworkXError("node %s not in graph" % (n,))
-        info += "Node % s has the following properties:\n" % n
-        info += "Degree: %d\n" % G.degree(n)
-        info += "Neighbors: "
-        info += ' '.join(str(nbr) for nbr in G.neighbors(n))
-    return info
-
-
 def set_node_attributes(G, values, name=None):
     """Sets node attributes from a given value or dictionary of values.
 
@@ -608,8 +584,8 @@ def set_node_attributes(G, values, name=None):
         >>> bb = nx.betweenness_centrality(G)
         >>> isinstance(bb, dict)
         True
-        >>> nx.set_node_attributes(G, bb, 'betweenness')
-        >>> G.nodes[1]['betweenness']
+        >>> nx.set_node_attributes(G, bb, "betweenness")
+        >>> G.nodes[1]["betweenness"]
         1.0
 
     If you provide a list as the second argument, updates to the list
@@ -617,13 +593,13 @@ def set_node_attributes(G, values, name=None):
 
         >>> G = nx.path_graph(3)
         >>> labels = []
-        >>> nx.set_node_attributes(G, labels, 'labels')
-        >>> labels.append('foo')
-        >>> G.nodes[0]['labels']
+        >>> nx.set_node_attributes(G, labels, "labels")
+        >>> labels.append("foo")
+        >>> G.nodes[0]["labels"]
         ['foo']
-        >>> G.nodes[1]['labels']
+        >>> G.nodes[1]["labels"]
         ['foo']
-        >>> G.nodes[2]['labels']
+        >>> G.nodes[2]["labels"]
         ['foo']
 
     If you provide a dictionary of dictionaries as the second argument,
@@ -631,17 +607,28 @@ def set_node_attributes(G, values, name=None):
     dictionary of node attributes for that node::
 
         >>> G = nx.path_graph(3)
-        >>> attrs = {0: {'attr1': 20, 'attr2': 'nothing'}, 1: {'attr2': 3}}
+        >>> attrs = {0: {"attr1": 20, "attr2": "nothing"}, 1: {"attr2": 3}}
         >>> nx.set_node_attributes(G, attrs)
-        >>> G.nodes[0]['attr1']
+        >>> G.nodes[0]["attr1"]
         20
-        >>> G.nodes[0]['attr2']
+        >>> G.nodes[0]["attr2"]
         'nothing'
-        >>> G.nodes[1]['attr2']
+        >>> G.nodes[1]["attr2"]
         3
         >>> G.nodes[2]
         {}
 
+    Note that if the dictionary contains nodes that are not in `G`, the
+    values are silently ignored::
+
+        >>> G = nx.Graph()
+        >>> G.add_node(0)
+        >>> nx.set_node_attributes(G, {0: "red", 1: "blue"}, name="color")
+        >>> G.nodes[0]["color"]
+        'red'
+        >>> 1 in G.nodes
+        False
+
     """
     # Set node attributes based on type of `values`
     if name is not None:  # `values` must not be a dict of dict
@@ -662,7 +649,7 @@ def set_node_attributes(G, values, name=None):
                 pass
 
 
-def get_node_attributes(G, name):
+def get_node_attributes(G, name, default=None):
     """Get node attributes from graph
 
     Parameters
@@ -672,6 +659,11 @@ def get_node_attributes(G, name):
     name : string
        Attribute name
 
+    default: object (default=None)
+       Default value of the node attribute if there is no value set for that
+       node in graph. If `None` then nodes without this attribute are not
+       included in the returned dict.
+
     Returns
     -------
     Dictionary of attributes keyed by node.
@@ -679,11 +671,17 @@ def get_node_attributes(G, name):
     Examples
     --------
     >>> G = nx.Graph()
-    >>> G.add_nodes_from([1, 2, 3], color='red')
-    >>> color = nx.get_node_attributes(G, 'color')
+    >>> G.add_nodes_from([1, 2, 3], color="red")
+    >>> color = nx.get_node_attributes(G, "color")
     >>> color[1]
     'red'
+    >>> G.add_node(4)
+    >>> color = nx.get_node_attributes(G, "color", default="yellow")
+    >>> color[4]
+    'yellow'
     """
+    if default is not None:
+        return {n: d.get(name, default) for n, d in G.nodes.items()}
     return {n: d[name] for n, d in G.nodes.items() if name in d}
 
 
@@ -723,35 +721,70 @@ def set_edge_attributes(G, values, name=None):
 
         >>> G = nx.path_graph(3)
         >>> bb = nx.edge_betweenness_centrality(G, normalized=False)
-        >>> nx.set_edge_attributes(G, bb, 'betweenness')
-        >>> G.edges[1, 2]['betweenness']
+        >>> nx.set_edge_attributes(G, bb, "betweenness")
+        >>> G.edges[1, 2]["betweenness"]
         2.0
 
     If you provide a list as the second argument, updates to the list
     will be reflected in the edge attribute for each edge::
 
         >>> labels = []
-        >>> nx.set_edge_attributes(G, labels, 'labels')
-        >>> labels.append('foo')
-        >>> G.edges[0, 1]['labels']
+        >>> nx.set_edge_attributes(G, labels, "labels")
+        >>> labels.append("foo")
+        >>> G.edges[0, 1]["labels"]
         ['foo']
-        >>> G.edges[1, 2]['labels']
+        >>> G.edges[1, 2]["labels"]
         ['foo']
 
     If you provide a dictionary of dictionaries as the second argument,
     the entire dictionary will be used to update edge attributes::
 
         >>> G = nx.path_graph(3)
-        >>> attrs = {(0, 1): {'attr1': 20, 'attr2': 'nothing'},
-        ...          (1, 2): {'attr2': 3}}
+        >>> attrs = {(0, 1): {"attr1": 20, "attr2": "nothing"}, (1, 2): {"attr2": 3}}
         >>> nx.set_edge_attributes(G, attrs)
-        >>> G[0][1]['attr1']
+        >>> G[0][1]["attr1"]
         20
-        >>> G[0][1]['attr2']
+        >>> G[0][1]["attr2"]
         'nothing'
-        >>> G[1][2]['attr2']
+        >>> G[1][2]["attr2"]
         3
 
+    The attributes of one Graph can be used to set those of another.
+
+        >>> H = nx.path_graph(3)
+        >>> nx.set_edge_attributes(H, G.edges)
+
+    Note that if the dict contains edges that are not in `G`, they are
+    silently ignored::
+
+        >>> G = nx.Graph([(0, 1)])
+        >>> nx.set_edge_attributes(G, {(1, 2): {"weight": 2.0}})
+        >>> (1, 2) in G.edges()
+        False
+
+    For multigraphs, the `values` dict is expected to be keyed by 3-tuples
+    including the edge key::
+
+        >>> MG = nx.MultiGraph()
+        >>> edges = [(0, 1), (0, 1)]
+        >>> MG.add_edges_from(edges)  # Returns list of edge keys
+        [0, 1]
+        >>> attributes = {(0, 1, 0): {"cost": 21}, (0, 1, 1): {"cost": 7}}
+        >>> nx.set_edge_attributes(MG, attributes)
+        >>> MG[0][1][0]["cost"]
+        21
+        >>> MG[0][1][1]["cost"]
+        7
+
+    If MultiGraph attributes are desired for a Graph, you must convert the 3-tuple
+    multiedge to a 2-tuple edge and the last multiedge's attribute value will
+    overwrite the previous values. Continuing from the previous case we get::
+
+        >>> H = nx.path_graph([0, 1, 2])
+        >>> nx.set_edge_attributes(H, {(u, v): ed for u, v, ed in MG.edges.data()})
+        >>> nx.get_edge_attributes(H, "cost")
+        {(0, 1): 7}
+
     """
     if name is not None:
         # `values` does not contain attribute names
@@ -789,7 +822,7 @@ def set_edge_attributes(G, values, name=None):
                     pass
 
 
-def get_edge_attributes(G, name):
+def get_edge_attributes(G, name, default=None):
     """Get edge attributes from graph
 
     Parameters
@@ -799,6 +832,11 @@ def get_edge_attributes(G, name):
     name : string
        Attribute name
 
+    default: object (default=None)
+       Default value of the edge attribute if there is no value set for that
+       edge in graph. If `None` then edges without this attribute are not
+       included in the returned dict.
+
     Returns
     -------
     Dictionary of attributes keyed by edge. For (di)graphs, the keys are
@@ -808,15 +846,21 @@ def get_edge_attributes(G, name):
     Examples
     --------
     >>> G = nx.Graph()
-    >>> nx.add_path(G, [1, 2, 3], color='red')
-    >>> color = nx.get_edge_attributes(G, 'color')
+    >>> nx.add_path(G, [1, 2, 3], color="red")
+    >>> color = nx.get_edge_attributes(G, "color")
     >>> color[(1, 2)]
     'red'
+    >>> G.add_edge(3, 4)
+    >>> color = nx.get_edge_attributes(G, "color", default="yellow")
+    >>> color[(3, 4)]
+    'yellow'
     """
     if G.is_multigraph():
         edges = G.edges(keys=True, data=True)
     else:
         edges = G.edges(data=True)
+    if default is not None:
+        return {x[:-1]: x[-1].get(name, default) for x in edges}
     return {x[:-1]: x[-1][name] for x in edges if name in x[-1]}
 
 
@@ -866,12 +910,12 @@ def non_neighbors(graph, node):
 
 
 def non_edges(graph):
-    """Returns the non-existent edges in the graph.
+    """Returns the nonexistent edges in the graph.
 
     Parameters
     ----------
     graph : NetworkX graph.
-        Graph to find non-existent edges.
+        Graph to find nonexistent edges.
 
     Returns
     -------
@@ -890,9 +934,9 @@ def non_edges(graph):
                 yield (u, v)
 
 
-@not_implemented_for('directed')
+@not_implemented_for("directed")
 def common_neighbors(G, u, v):
-    """Return the common neighbors of two nodes in a graph.
+    """Returns the common neighbors of two nodes in a graph.
 
     Parameters
     ----------
@@ -919,16 +963,16 @@ def common_neighbors(G, u, v):
     [2, 3, 4]
     """
     if u not in G:
-        raise nx.NetworkXError('u is not in the graph.')
+        raise nx.NetworkXError("u is not in the graph.")
     if v not in G:
-        raise nx.NetworkXError('v is not in the graph.')
+        raise nx.NetworkXError("v is not in the graph.")
 
     # Return a generator explicitly instead of yielding so that the above
     # checks are executed eagerly.
     return (w for w in G[u] if w in G[v] and w not in (u, v))
 
 
-def is_weighted(G, edge=None, weight='weight'):
+def is_weighted(G, edge=None, weight="weight"):
     """Returns True if `G` has weighted edges.
 
     Parameters
@@ -970,7 +1014,7 @@ def is_weighted(G, edge=None, weight='weight'):
     if edge is not None:
         data = G.get_edge_data(*edge)
         if data is None:
-            msg = 'Edge {!r} does not exist.'.format(edge)
+            msg = f"Edge {edge!r} does not exist."
             raise nx.NetworkXError(msg)
         return weight in data
 
@@ -981,7 +1025,7 @@ def is_weighted(G, edge=None, weight='weight'):
     return all(weight in data for u, v, data in G.edges(data=True))
 
 
-def is_negatively_weighted(G, edge=None, weight='weight'):
+def is_negatively_weighted(G, edge=None, weight="weight"):
     """Returns True if `G` has negatively weighted edges.
 
     Parameters
@@ -1014,11 +1058,11 @@ def is_negatively_weighted(G, edge=None, weight='weight'):
     >>> G.add_edge(1, 2, weight=4)
     >>> nx.is_negatively_weighted(G, (1, 2))
     False
-    >>> G[2][4]['weight'] = -2
+    >>> G[2][4]["weight"] = -2
     >>> nx.is_negatively_weighted(G)
     True
     >>> G = nx.DiGraph()
-    >>> edges = [('0', '3', 3), ('0', '1', -5), ('1', '0', -2)]
+    >>> edges = [("0", "3", 3), ("0", "1", -5), ("1", "0", -2)]
     >>> G.add_weighted_edges_from(edges)
     >>> nx.is_negatively_weighted(G)
     True
@@ -1027,12 +1071,11 @@ def is_negatively_weighted(G, edge=None, weight='weight'):
     if edge is not None:
         data = G.get_edge_data(*edge)
         if data is None:
-            msg = 'Edge {!r} does not exist.'.format(edge)
+            msg = f"Edge {edge!r} does not exist."
             raise nx.NetworkXError(msg)
         return weight in data and data[weight] < 0
 
-    return any(weight in data and data[weight] < 0
-               for u, v, data in G.edges(data=True))
+    return any(weight in data and data[weight] < 0 for u, v, data in G.edges(data=True))
 
 
 def is_empty(G):
@@ -1075,7 +1118,7 @@ def nodes_with_selfloops(G):
 
     Examples
     --------
-    >>> G = nx.Graph()   # or DiGraph, MultiGraph, MultiDiGraph, etc
+    >>> G = nx.Graph()  # or DiGraph, MultiGraph, MultiDiGraph, etc
     >>> G.add_edge(1, 1)
     >>> G.add_edge(1, 2)
     >>> list(nx.nodes_with_selfloops(G))
@@ -1092,6 +1135,8 @@ def selfloop_edges(G, data=False, keys=False, default=None):
 
     Parameters
     ----------
+    G : graph
+        A NetworkX graph.
     data : string or bool, optional (default=False)
         Return selfloop edges as two tuples (u, v) (data=False)
         or three-tuples (u, v, datadict) (data=True)
@@ -1113,7 +1158,7 @@ def selfloop_edges(G, data=False, keys=False, default=None):
 
     Examples
     --------
-    >>> G = nx.MultiGraph()   # or Graph, DiGraph, MultiDiGraph, etc
+    >>> G = nx.MultiGraph()  # or Graph, DiGraph, MultiDiGraph, etc
     >>> ekey = G.add_edge(1, 1)
     >>> ekey = G.add_edge(1, 2)
     >>> list(nx.selfloop_edges(G))
@@ -1128,44 +1173,62 @@ def selfloop_edges(G, data=False, keys=False, default=None):
     if data is True:
         if G.is_multigraph():
             if keys is True:
-                return ((n, n, k, d)
-                        for n, nbrs in G.adj.items()
-                        if n in nbrs for k, d in nbrs[n].items())
+                return (
+                    (n, n, k, d)
+                    for n, nbrs in G.adj.items()
+                    if n in nbrs
+                    for k, d in nbrs[n].items()
+                )
             else:
-                return ((n, n, d)
-                        for n, nbrs in G.adj.items()
-                        if n in nbrs for d in nbrs[n].values())
+                return (
+                    (n, n, d)
+                    for n, nbrs in G.adj.items()
+                    if n in nbrs
+                    for d in nbrs[n].values()
+                )
         else:
             return ((n, n, nbrs[n]) for n, nbrs in G.adj.items() if n in nbrs)
     elif data is not False:
         if G.is_multigraph():
             if keys is True:
-                return ((n, n, k, d.get(data, default))
-                        for n, nbrs in G.adj.items()
-                        if n in nbrs for k, d in nbrs[n].items())
+                return (
+                    (n, n, k, d.get(data, default))
+                    for n, nbrs in G.adj.items()
+                    if n in nbrs
+                    for k, d in nbrs[n].items()
+                )
             else:
-                return ((n, n, d.get(data, default))
-                        for n, nbrs in G.adj.items()
-                        if n in nbrs for d in nbrs[n].values())
+                return (
+                    (n, n, d.get(data, default))
+                    for n, nbrs in G.adj.items()
+                    if n in nbrs
+                    for d in nbrs[n].values()
+                )
         else:
-            return ((n, n, nbrs[n].get(data, default))
-                    for n, nbrs in G.adj.items() if n in nbrs)
+            return (
+                (n, n, nbrs[n].get(data, default))
+                for n, nbrs in G.adj.items()
+                if n in nbrs
+            )
     else:
         if G.is_multigraph():
             if keys is True:
-                return ((n, n, k)
-                        for n, nbrs in G.adj.items()
-                        if n in nbrs for k in nbrs[n])
+                return (
+                    (n, n, k) for n, nbrs in G.adj.items() if n in nbrs for k in nbrs[n]
+                )
             else:
-                return ((n, n)
-                        for n, nbrs in G.adj.items()
-                        if n in nbrs for d in nbrs[n].values())
+                return (
+                    (n, n)
+                    for n, nbrs in G.adj.items()
+                    if n in nbrs
+                    for i in range(len(nbrs[n]))  # for easy edge removal (#4068)
+                )
         else:
             return ((n, n) for n, nbrs in G.adj.items() if n in nbrs)
 
 
 def number_of_selfloops(G):
-    """Return the number of selfloop edges.
+    """Returns the number of selfloop edges.
 
     A selfloop edge has the same node at both ends.
 
@@ -1180,10 +1243,71 @@ def number_of_selfloops(G):
 
     Examples
     --------
-    >>> G = nx.Graph()   # or DiGraph, MultiGraph, MultiDiGraph, etc
+    >>> G = nx.Graph()  # or DiGraph, MultiGraph, MultiDiGraph, etc
     >>> G.add_edge(1, 1)
     >>> G.add_edge(1, 2)
     >>> nx.number_of_selfloops(G)
     1
     """
     return sum(1 for _ in nx.selfloop_edges(G))
+
+
+def is_path(G, path):
+    """Returns whether or not the specified path exists.
+
+    For it to return True, every node on the path must exist and
+    each consecutive pair must be connected via one or more edges.
+
+    Parameters
+    ----------
+    G : graph
+        A NetworkX graph.
+
+    path : list
+        A list of nodes which defines the path to traverse
+
+    Returns
+    -------
+    bool
+        True if `path` is a valid path in `G`
+
+    """
+    return all((node in G and nbr in G[node]) for node, nbr in nx.utils.pairwise(path))
+
+
+def path_weight(G, path, weight):
+    """Returns total cost associated with specified path and weight
+
+    Parameters
+    ----------
+    G : graph
+        A NetworkX graph.
+
+    path: list
+        A list of node labels which defines the path to traverse
+
+    weight: string
+        A string indicating which edge attribute to use for path cost
+
+    Returns
+    -------
+    cost: int or float
+        An integer or a float representing the total cost with respect to the
+        specified weight of the specified path
+
+    Raises
+    ------
+    NetworkXNoPath
+        If the specified edge does not exist.
+    """
+    multigraph = G.is_multigraph()
+    cost = 0
+
+    if not nx.is_path(G, path):
+        raise nx.NetworkXNoPath("path does not exist")
+    for node, nbr in nx.utils.pairwise(path):
+        if multigraph:
+            cost += min(v[weight] for v in G[node][nbr].values())
+        else:
+            cost += G[node][nbr][weight]
+    return cost
