@@ -1,82 +1,119 @@
-# -*- coding: utf-8 -*-
-#    Copyright (C) 2004-2018 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
-#
-# Authors: Aric Hagberg (hagberg@lanl.gov)
-#          Dan Schult (dschult@colgate.edu)
-#          Ben Edwards (BJEdwards@gmail.com)
-#          Arya McCarthy (admccarthy@smu.edu)
-#          Cole MacLean (maclean.cole@gmail.com)
-
 """Generators for geometric graphs.
 """
-from __future__ import division
 
-from bisect import bisect_left
-from itertools import combinations
-from itertools import product
-from math import sqrt
 import math
-try:
-    from scipy.spatial import cKDTree as KDTree
-except ImportError:
-    _is_scipy_available = False
-else:
-    _is_scipy_available = True
+from bisect import bisect_left
+from itertools import accumulate, combinations, product
 
 import networkx as nx
-from networkx.utils import nodes_or_number, py_random_state
+from networkx.utils import py_random_state
 
-__all__ = ['geographical_threshold_graph', 'waxman_graph',
-           'navigable_small_world_graph', 'random_geometric_graph',
-           'soft_random_geometric_graph', 'thresholded_random_geometric_graph']
+__all__ = [
+    "geometric_edges",
+    "geographical_threshold_graph",
+    "navigable_small_world_graph",
+    "random_geometric_graph",
+    "soft_random_geometric_graph",
+    "thresholded_random_geometric_graph",
+    "waxman_graph",
+]
 
 
-def euclidean(x, y):
-    """Returns the Euclidean distance between the vectors ``x`` and ``y``.
+@nx._dispatch(node_attrs="pos_name")
+def geometric_edges(G, radius, p=2, *, pos_name="pos"):
+    """Returns edge list of node pairs within `radius` of each other.
 
-    Each of ``x`` and ``y`` can be any iterable of numbers. The
-    iterables must be of the same length.
+    Parameters
+    ----------
+    G : networkx graph
+        The graph from which to generate the edge list. The nodes in `G` should
+        have an attribute ``pos`` corresponding to the node position, which is
+        used to compute the distance to other nodes.
+    radius : scalar
+        The distance threshold. Edges are included in the edge list if the
+        distance between the two nodes is less than `radius`.
+    pos_name : string, default="pos"
+        The name of the node attribute which represents the position of each
+        node in 2D coordinates. Every node in the Graph must have this attribute.
+    p : scalar, default=2
+        The `Minkowski distance metric
+        <https://en.wikipedia.org/wiki/Minkowski_distance>`_ used to compute
+        distances. The default value is 2, i.e. Euclidean distance.
 
+    Returns
+    -------
+    edges : list
+        List of edges whose distances are less than `radius`
+
+    Notes
+    -----
+    Radius uses Minkowski distance metric `p`.
+    If scipy is available, `scipy.spatial.cKDTree` is used to speed computation.
+
+    Examples
+    --------
+    Create a graph with nodes that have a "pos" attribute representing 2D
+    coordinates.
+
+    >>> G = nx.Graph()
+    >>> G.add_nodes_from([
+    ...     (0, {"pos": (0, 0)}),
+    ...     (1, {"pos": (3, 0)}),
+    ...     (2, {"pos": (8, 0)}),
+    ... ])
+    >>> nx.geometric_edges(G, radius=1)
+    []
+    >>> nx.geometric_edges(G, radius=4)
+    [(0, 1)]
+    >>> nx.geometric_edges(G, radius=6)
+    [(0, 1), (1, 2)]
+    >>> nx.geometric_edges(G, radius=9)
+    [(0, 1), (0, 2), (1, 2)]
     """
-    return sqrt(sum((a - b) ** 2 for a, b in zip(x, y)))
+    # Input validation - every node must have a "pos" attribute
+    for n, pos in G.nodes(data=pos_name):
+        if pos is None:
+            raise nx.NetworkXError(
+                f"Node {n} (and all nodes) must have a '{pos_name}' attribute."
+            )
+
+    # NOTE: See _geometric_edges for the actual implementation. The reason this
+    # is split into two functions is to avoid the overhead of input validation
+    # every time the function is called internally in one of the other
+    # geometric generators
+    return _geometric_edges(G, radius, p, pos_name)
 
 
-def _fast_edges(G, radius, p):
-    """Returns edge list of node pairs within `radius` of each other
-       using scipy KDTree and Minkowski distance metric `p`
-
-    Requires scipy to be installed.
+def _geometric_edges(G, radius, p, pos_name):
     """
-    pos = nx.get_node_attributes(G, 'pos')
-    nodes, coords = list(zip(*pos.items()))
-    kdtree = KDTree(coords)  # Cannot provide generator.
+    Implements `geometric_edges` without input validation. See `geometric_edges`
+    for complete docstring.
+    """
+    nodes_pos = G.nodes(data=pos_name)
+    try:
+        import scipy as sp
+    except ImportError:
+        # no scipy KDTree so compute by for-loop
+        radius_p = radius**p
+        edges = [
+            (u, v)
+            for (u, pu), (v, pv) in combinations(nodes_pos, 2)
+            if sum(abs(a - b) ** p for a, b in zip(pu, pv)) <= radius_p
+        ]
+        return edges
+    # scipy KDTree is available
+    nodes, coords = list(zip(*nodes_pos))
+    kdtree = sp.spatial.cKDTree(coords)  # Cannot provide generator.
     edge_indexes = kdtree.query_pairs(radius, p)
-    edges = ((nodes[u], nodes[v]) for u, v in edge_indexes)
-    return edges
-
-
-def _slow_edges(G, radius, p):
-    """Returns edge list of node pairs within `radius` of each other
-       using Minkowski distance metric `p`
-
-    Works without scipy, but in `O(n^2)` time.
-    """
-    # TODO This can be parallelized.
-    edges = []
-    for (u, pu), (v, pv) in combinations(G.nodes(data='pos'), 2):
-        if sum(abs(a - b) ** p for a, b in zip(pu, pv)) <= radius ** p:
-            edges.append((u, v))
+    edges = [(nodes[u], nodes[v]) for u, v in sorted(edge_indexes)]
     return edges
 
 
 @py_random_state(5)
-@nodes_or_number(0)
-def random_geometric_graph(n, radius, dim=2, pos=None, p=2, seed=None):
+@nx._dispatch(graphs=None)
+def random_geometric_graph(
+    n, radius, dim=2, pos=None, p=2, seed=None, *, pos_name="pos"
+):
     """Returns a random geometric graph in the unit cube of dimensions `dim`.
 
     The random geometric graph model places `n` nodes uniformly at
@@ -107,6 +144,9 @@ def random_geometric_graph(n, radius, dim=2, pos=None, p=2, seed=None):
     seed : integer, random_state, or None (default)
         Indicator of random number generation state.
         See :ref:`Randomness<randomness>`.
+    pos_name : string, default="pos"
+        The name of the node attribute which represents the position
+        in 2D coordinates of the node in the returned graph.
 
     Returns
     -------
@@ -148,34 +188,27 @@ def random_geometric_graph(n, radius, dim=2, pos=None, p=2, seed=None):
     # TODO Is this function just a special case of the geographical
     # threshold graph?
     #
-    #     n_name, nodes = n
-    #     half_radius = {v: radius / 2 for v in nodes}
+    #     half_radius = {v: radius / 2 for v in n}
     #     return geographical_threshold_graph(nodes, theta=1, alpha=1,
     #                                         weight=half_radius)
     #
-    n_name, nodes = n
-    G = nx.Graph()
-    G.add_nodes_from(nodes)
+    G = nx.empty_graph(n)
     # If no positions are provided, choose uniformly random vectors in
     # Euclidean space of the specified dimension.
     if pos is None:
-        pos = {v: [seed.random() for i in range(dim)] for v in nodes}
-    nx.set_node_attributes(G, pos, 'pos')
-
-    if _is_scipy_available:
-        edges = _fast_edges(G, radius, p)
-    else:
-        edges = _slow_edges(G, radius, p)
-    G.add_edges_from(edges)
+        pos = {v: [seed.random() for i in range(dim)] for v in G}
+    nx.set_node_attributes(G, pos, pos_name)
 
+    G.add_edges_from(_geometric_edges(G, radius, p, pos_name))
     return G
 
 
 @py_random_state(6)
-@nodes_or_number(0)
-def soft_random_geometric_graph(n, radius, dim=2, pos=None, p=2, p_dist=None,
-                                seed=None):
-    """Returns a soft random geometric graph in the unit cube.
+@nx._dispatch(graphs=None)
+def soft_random_geometric_graph(
+    n, radius, dim=2, pos=None, p=2, p_dist=None, seed=None, *, pos_name="pos"
+):
+    r"""Returns a soft random geometric graph in the unit cube.
 
     The soft random geometric graph [1] model places `n` nodes uniformly at
     random in the unit cube in dimension `dim`. Two nodes of distance, `dist`,
@@ -220,6 +253,9 @@ def soft_random_geometric_graph(n, radius, dim=2, pos=None, p=2, p_dist=None,
     seed : integer, random_state, or None (default)
         Indicator of random number generation state.
         See :ref:`Randomness<randomness>`.
+    pos_name : string, default="pos"
+        The name of the node attribute which represents the position
+        in 2D coordinates of the node in the returned graph.
 
     Returns
     -------
@@ -262,26 +298,24 @@ def soft_random_geometric_graph(n, radius, dim=2, pos=None, p=2, p_dist=None,
     >>> import math
     >>> n = 100
     >>> pos = {i: (random.gauss(0, 2), random.gauss(0, 2)) for i in range(n)}
-    >>> p_dist = lambda dist : math.exp(-dist)
+    >>> p_dist = lambda dist: math.exp(-dist)
     >>> G = nx.soft_random_geometric_graph(n, 0.2, pos=pos, p_dist=p_dist)
 
     References
     ----------
     .. [1] Penrose, Mathew D. "Connectivity of soft random geometric graphs."
            The Annals of Applied Probability 26.2 (2016): 986-1028.
-       [2] scipy.stats -
+    .. [2] scipy.stats -
            https://docs.scipy.org/doc/scipy/reference/tutorial/stats.html
 
     """
-    n_name, nodes = n
-    G = nx.Graph()
-    G.name = 'soft_random_geometric_graph({}, {}, {})'.format(n, radius, dim)
-    G.add_nodes_from(nodes)
+    G = nx.empty_graph(n)
+    G.name = f"soft_random_geometric_graph({n}, {radius}, {dim})"
     # If no positions are provided, choose uniformly random vectors in
     # Euclidean space of the specified dimension.
     if pos is None:
-        pos = {v: [seed.random() for i in range(dim)] for v in nodes}
-    nx.set_node_attributes(G, pos, 'pos')
+        pos = {v: [seed.random() for i in range(dim)] for v in G}
+    nx.set_node_attributes(G, pos, pos_name)
 
     # if p_dist function not supplied the default function is an exponential
     # distribution with rate parameter :math:`\lambda=1`.
@@ -290,32 +324,30 @@ def soft_random_geometric_graph(n, radius, dim=2, pos=None, p=2, p_dist=None,
         def p_dist(dist):
             return math.exp(-dist)
 
-    def should_join(pair):
-        u, v = pair
-        u_pos, v_pos = pos[u], pos[v]
-        dist = (sum(abs(a - b) ** p for a, b in zip(u_pos, v_pos)))**(1 / p)
-        # Check if dist <= radius parameter. This check is redundant if scipy
-        # is available and _fast_edges routine is used, but provides the
-        # check in case scipy is not available and all edge combinations
-        # need to be checked
-        if dist <= radius:
-            return seed.random() < p_dist(dist)
-        else:
-            return False
-
-    if _is_scipy_available:
-        edges = _fast_edges(G, radius, p)
-        G.add_edges_from(filter(should_join, edges))
-    else:
-        G.add_edges_from(filter(should_join, combinations(G, 2)))
+    def should_join(edge):
+        u, v = edge
+        dist = (sum(abs(a - b) ** p for a, b in zip(pos[u], pos[v]))) ** (1 / p)
+        return seed.random() < p_dist(dist)
 
+    G.add_edges_from(filter(should_join, _geometric_edges(G, radius, p, pos_name)))
     return G
 
 
 @py_random_state(7)
-@nodes_or_number(0)
-def geographical_threshold_graph(n, theta, dim=2, pos=None, weight=None,
-                                 metric=None, p_dist=None, seed=None):
+@nx._dispatch(graphs=None)
+def geographical_threshold_graph(
+    n,
+    theta,
+    dim=2,
+    pos=None,
+    weight=None,
+    metric=None,
+    p_dist=None,
+    seed=None,
+    *,
+    pos_name="pos",
+    weight_name="weight",
+):
     r"""Returns a geographical threshold graph.
 
     The geographical threshold graph model places $n$ nodes uniformly at
@@ -324,11 +356,13 @@ def geographical_threshold_graph(n, theta, dim=2, pos=None, weight=None,
 
     .. math::
 
-       (w_u + w_v)h(r) \ge \theta
+       (w_u + w_v)p_{dist}(r) \ge \theta
 
-    where `r` is the distance between `u` and `v`, h(r) is a probability of
-    connection as a function of `r`, and :math:`\theta` as the threshold
-    parameter. h(r) corresponds to the p_dist parameter.
+    where `r` is the distance between `u` and `v`, `p_dist` is any function of
+    `r`, and :math:`\theta` as the threshold parameter. `p_dist` is used to
+    give weight to the distance between nodes when deciding whether or not
+    they should be connected. The larger `p_dist` is, the more prone nodes
+    separated by `r` are to be connected, and vice versa.
 
     Parameters
     ----------
@@ -360,20 +394,26 @@ def geographical_threshold_graph(n, theta, dim=2, pos=None, weight=None,
 
         .. _metric: https://en.wikipedia.org/wiki/Metric_%28mathematics%29
     p_dist : function, optional
-        A probability density function computing the probability of
-        connecting two nodes that are of distance, r, computed by metric.
-        The probability density function, `p_dist`, must
-        be any function that takes the metric value as input
-        and outputs a single probability value between 0-1.
-        The scipy.stats package has many probability distribution functions
-        implemented and tools for custom probability distribution
-        definitions [2], and passing the .pdf method of scipy.stats
-        distributions can be used here. If the probability
-        function, `p_dist`, is not supplied, the default exponential function
-        :math: `r^{-2}` is used.
+        Any function used to give weight to the distance between nodes when
+        deciding whether or not they should be connected. `p_dist` was
+        originally conceived as a probability density function giving the
+        probability of connecting two nodes that are of metric distance `r`
+        apart. The implementation here allows for more arbitrary definitions
+        of `p_dist` that do not need to correspond to valid probability
+        density functions. The :mod:`scipy.stats` package has many
+        probability density functions implemented and tools for custom
+        probability density definitions, and passing the ``.pdf`` method of
+        scipy.stats distributions can be used here. If ``p_dist=None``
+        (the default), the exponential function :math:`r^{-2}` is used.
     seed : integer, random_state, or None (default)
         Indicator of random number generation state.
         See :ref:`Randomness<randomness>`.
+    pos_name : string, default="pos"
+        The name of the node attribute which represents the position
+        in 2D coordinates of the node in the returned graph.
+    weight_name : string, default="weight"
+        The name of the node attribute which represents the weight
+        of the node in the returned graph.
 
     Returns
     -------
@@ -415,16 +455,6 @@ def geographical_threshold_graph(n, theta, dim=2, pos=None, weight=None,
     If node positions are not specified they are randomly assigned from the
     uniform distribution.
 
-    Starting in NetworkX 2.1 the parameter ``alpha`` is deprecated and replaced
-    with the customizable ``p_dist`` function parameter, which defaults to r^-2
-    if ``p_dist`` is not supplied. To reproduce networks of earlier NetworkX
-    versions, a custom function needs to be defined and passed as the
-    ``p_dist`` parameter. For example, if the parameter ``alpha`` = 2 was used
-    in NetworkX 2.0, the custom function def custom_dist(r): r**-2 can be
-    passed in versions >=2.1 as the parameter p_dist = custom_dist to
-    produce an equivalent network. Note the change in sign from +2 to -2 in
-    this parameter change.
-
     References
     ----------
     .. [1] Masuda, N., Miwa, H., Konno, N.:
@@ -436,9 +466,7 @@ def geographical_threshold_graph(n, theta, dim=2, pos=None, weight=None,
        in Algorithms and Models for the Web-Graph (WAW 2007),
        Antony Bonato and Fan Chung (Eds), pp. 209--216, 2007
     """
-    n_name, nodes = n
-    G = nx.Graph()
-    G.add_nodes_from(nodes)
+    G = nx.empty_graph(n)
     # If no weights are provided, choose them from an exponential
     # distribution.
     if weight is None:
@@ -446,15 +474,16 @@ def geographical_threshold_graph(n, theta, dim=2, pos=None, weight=None,
     # If no positions are provided, choose uniformly random vectors in
     # Euclidean space of the specified dimension.
     if pos is None:
-        pos = {v: [seed.random() for i in range(dim)] for v in nodes}
+        pos = {v: [seed.random() for i in range(dim)] for v in G}
     # If no distance metric is provided, use Euclidean distance.
     if metric is None:
-        metric = euclidean
-    nx.set_node_attributes(G, weight, 'weight')
-    nx.set_node_attributes(G, pos, 'pos')
+        metric = math.dist
+    nx.set_node_attributes(G, weight, weight_name)
+    nx.set_node_attributes(G, pos, pos_name)
 
     # if p_dist is not supplied, use default r^-2
     if p_dist is None:
+
         def p_dist(r):
             return r**-2
 
@@ -472,10 +501,19 @@ def geographical_threshold_graph(n, theta, dim=2, pos=None, weight=None,
 
 
 @py_random_state(6)
-@nodes_or_number(0)
-def waxman_graph(n, beta=0.4, alpha=0.1, L=None, domain=(0, 0, 1, 1),
-                 metric=None, seed=None):
-    r"""Return a Waxman random graph.
+@nx._dispatch(graphs=None)
+def waxman_graph(
+    n,
+    beta=0.4,
+    alpha=0.1,
+    L=None,
+    domain=(0, 0, 1, 1),
+    metric=None,
+    seed=None,
+    *,
+    pos_name="pos",
+):
+    r"""Returns a Waxman random graph.
 
     The Waxman random graph model places `n` nodes uniformly at random
     in a rectangular domain. Each pair of nodes at distance `d` is
@@ -527,6 +565,9 @@ def waxman_graph(n, beta=0.4, alpha=0.1, L=None, domain=(0, 0, 1, 1),
     seed : integer, random_state, or None (default)
         Indicator of random number generation state.
         See :ref:`Randomness<randomness>`.
+    pos_name : string, default="pos"
+        The name of the node attribute which represents the position
+        in 2D coordinates of the node in the returned graph.
 
     Returns
     -------
@@ -559,16 +600,14 @@ def waxman_graph(n, beta=0.4, alpha=0.1, L=None, domain=(0, 0, 1, 1),
     .. [1]  B. M. Waxman, *Routing of multipoint connections*.
        IEEE J. Select. Areas Commun. 6(9),(1988) 1617--1622.
     """
-    n_name, nodes = n
-    G = nx.Graph()
-    G.add_nodes_from(nodes)
+    G = nx.empty_graph(n)
     (xmin, ymin, xmax, ymax) = domain
     # Each node gets a uniformly random position in the given rectangle.
     pos = {v: (seed.uniform(xmin, xmax), seed.uniform(ymin, ymax)) for v in G}
-    nx.set_node_attributes(G, pos, 'pos')
+    nx.set_node_attributes(G, pos, pos_name)
     # If no distance metric is provided, use Euclidean distance.
     if metric is None:
-        metric = euclidean
+        metric = math.dist
     # If the maximum distance L is not specified (that is, we are in the
     # Waxman-1 model), then find the maximum distance between any pair
     # of nodes.
@@ -578,9 +617,13 @@ def waxman_graph(n, beta=0.4, alpha=0.1, L=None, domain=(0, 0, 1, 1),
     if L is None:
         L = max(metric(x, y) for x, y in combinations(pos.values(), 2))
 
-        def dist(u, v): return metric(pos[u], pos[v])
+        def dist(u, v):
+            return metric(pos[u], pos[v])
+
     else:
-        def dist(u, v): return seed.random() * L
+
+        def dist(u, v):
+            return seed.random() * L
 
     # `pair` is the pair of nodes to decide whether to join.
     def should_join(pair):
@@ -591,8 +634,9 @@ def waxman_graph(n, beta=0.4, alpha=0.1, L=None, domain=(0, 0, 1, 1),
 
 
 @py_random_state(5)
+@nx._dispatch(graphs=None)
 def navigable_small_world_graph(n, p=1, q=1, r=2, dim=2, seed=None):
-    r"""Return a navigable small-world graph.
+    r"""Returns a navigable small-world graph.
 
     A navigable small-world graph is a directed grid with additional long-range
     connections that are chosen randomly.
@@ -637,11 +681,11 @@ def navigable_small_world_graph(n, p=1, q=1, r=2, dim=2, seed=None):
     .. [1] J. Kleinberg. The small-world phenomenon: An algorithmic
        perspective. Proc. 32nd ACM Symposium on Theory of Computing, 2000.
     """
-    if (p < 1):
+    if p < 1:
         raise nx.NetworkXException("p must be >= 1")
-    if (q < 0):
+    if q < 0:
         raise nx.NetworkXException("q must be >= 0")
-    if (r < 0):
+    if r < 0:
         raise nx.NetworkXException("r must be >= 1")
 
     G = nx.DiGraph()
@@ -655,7 +699,7 @@ def navigable_small_world_graph(n, p=1, q=1, r=2, dim=2, seed=None):
             if d <= p:
                 G.add_edge(p1, p2)
             probs.append(d**-r)
-        cdf = list(nx.utils.accumulate(probs))
+        cdf = list(accumulate(probs))
         for _ in range(q):
             target = nodes[bisect_left(cdf, seed.uniform(0, cdf[-1]))]
             G.add_edge(p1, target)
@@ -663,10 +707,21 @@ def navigable_small_world_graph(n, p=1, q=1, r=2, dim=2, seed=None):
 
 
 @py_random_state(7)
-@nodes_or_number(0)
-def thresholded_random_geometric_graph(n, radius, theta, dim=2,
-                                       pos=None, weight=None, p=2, seed=None):
-    """Returns a thresholded random geometric graph in the unit cube.
+@nx._dispatch(graphs=None)
+def thresholded_random_geometric_graph(
+    n,
+    radius,
+    theta,
+    dim=2,
+    pos=None,
+    weight=None,
+    p=2,
+    seed=None,
+    *,
+    pos_name="pos",
+    weight_name="weight",
+):
+    r"""Returns a thresholded random geometric graph in the unit cube.
 
     The thresholded random geometric graph [1] model places `n` nodes
     uniformly at random in the unit cube of dimensions `dim`. Each node
@@ -694,7 +749,7 @@ def thresholded_random_geometric_graph(n, radius, theta, dim=2,
         A dictionary keyed by node with node positions as values.
     weight : dict, optional
         Node weights as a dictionary of numbers keyed by node.
-    p : float, optional
+    p : float, optional (default 2)
         Which Minkowski distance metric to use.  `p` has to meet the condition
         ``1 <= p <= infinity``.
 
@@ -706,6 +761,12 @@ def thresholded_random_geometric_graph(n, radius, theta, dim=2,
     seed : integer, random_state, or None (default)
         Indicator of random number generation state.
         See :ref:`Randomness<randomness>`.
+    pos_name : string, default="pos"
+        The name of the node attribute which represents the position
+        in 2D coordinates of the node in the returned graph.
+    weight_name : string, default="weight"
+        The name of the node attribute which represents the weight
+        of the node in the returned graph.
 
     Returns
     -------
@@ -762,12 +823,8 @@ def thresholded_random_geometric_graph(n, radius, theta, dim=2,
     .. [1] http://cole-maclean.github.io/blog/files/thesis.pdf
 
     """
-
-    n_name, nodes = n
-    G = nx.Graph()
-    namestr = 'thresholded_random_geometric_graph({}, {}, {}, {})'
-    G.name = namestr.format(n, radius, theta, dim)
-    G.add_nodes_from(nodes)
+    G = nx.empty_graph(n)
+    G.name = f"thresholded_random_geometric_graph({n}, {radius}, {theta}, {dim})"
     # If no weights are provided, choose them from an exponential
     # distribution.
     if weight is None:
@@ -775,34 +832,15 @@ def thresholded_random_geometric_graph(n, radius, theta, dim=2,
     # If no positions are provided, choose uniformly random vectors in
     # Euclidean space of the specified dimension.
     if pos is None:
-        pos = {v: [seed.random() for i in range(dim)] for v in nodes}
+        pos = {v: [seed.random() for i in range(dim)] for v in G}
     # If no distance metric is provided, use Euclidean distance.
+    nx.set_node_attributes(G, weight, weight_name)
+    nx.set_node_attributes(G, pos, pos_name)
 
-    nx.set_node_attributes(G, weight, 'weight')
-    nx.set_node_attributes(G, pos, 'pos')
-
-    # Returns ``True`` if and only if the nodes whose attributes are
-    # ``du`` and ``dv`` should be joined, according to the threshold
-    # condition and node pairs are within the maximum connection
-    # distance, ``radius``.
-    def should_join(pair):
-        u, v = pair
-        u_weight, v_weight = weight[u], weight[v]
-        u_pos, v_pos = pos[u], pos[v]
-        dist = (sum(abs(a - b) ** p for a, b in zip(u_pos, v_pos)))**(1 / p)
-        # Check if dist is <= radius parameter. This check is redundant if
-        # scipy is available and _fast_edges routine is used, but provides
-        # the check in case scipy is not available and all edge combinations
-        # need to be checked
-        if dist <= radius:
-            return theta <= u_weight + v_weight
-        else:
-            return False
-
-    if _is_scipy_available:
-        edges = _fast_edges(G, radius, p)
-        G.add_edges_from(filter(should_join, edges))
-    else:
-        G.add_edges_from(filter(should_join, combinations(G, 2)))
-
+    edges = (
+        (u, v)
+        for u, v in _geometric_edges(G, radius, p, pos_name)
+        if weight[u] + weight[v] >= theta
+    )
+    G.add_edges_from(edges)
     return G
