@@ -37,6 +37,7 @@ __all__ = [
 
 
 @py_random_state(2)
+@nx._dispatch(graphs=None)
 def fast_gnp_random_graph(n, p, seed=None, directed=False):
     """Returns a $G_{n,p}$ random graph, also known as an Erdős-Rényi graph or
     a binomial graph.
@@ -108,6 +109,7 @@ def fast_gnp_random_graph(n, p, seed=None, directed=False):
 
 
 @py_random_state(2)
+@nx._dispatch(graphs=None)
 def gnp_random_graph(n, p, seed=None, directed=False):
     """Returns a $G_{n,p}$ random graph, also known as an Erdős-Rényi graph
     or a binomial graph.
@@ -172,6 +174,7 @@ erdos_renyi_graph = gnp_random_graph
 
 
 @py_random_state(2)
+@nx._dispatch(graphs=None)
 def dense_gnm_random_graph(n, m, seed=None):
     """Returns a $G_{n,m}$ random graph.
 
@@ -233,6 +236,7 @@ def dense_gnm_random_graph(n, m, seed=None):
 
 
 @py_random_state(2)
+@nx._dispatch(graphs=None)
 def gnm_random_graph(n, m, seed=None, directed=False):
     """Returns a $G_{n,m}$ random graph.
 
@@ -288,6 +292,7 @@ def gnm_random_graph(n, m, seed=None, directed=False):
 
 
 @py_random_state(3)
+@nx._dispatch(graphs=None)
 def newman_watts_strogatz_graph(n, k, p, seed=None):
     """Returns a Newman–Watts–Strogatz small-world graph.
 
@@ -358,6 +363,7 @@ def newman_watts_strogatz_graph(n, k, p, seed=None):
 
 
 @py_random_state(3)
+@nx._dispatch(graphs=None)
 def watts_strogatz_graph(n, k, p, seed=None):
     """Returns a Watts–Strogatz small-world graph.
 
@@ -432,6 +438,7 @@ def watts_strogatz_graph(n, k, p, seed=None):
 
 
 @py_random_state(4)
+@nx._dispatch(graphs=None)
 def connected_watts_strogatz_graph(n, k, p, tries=100, seed=None):
     """Returns a connected Watts–Strogatz small-world graph.
 
@@ -484,6 +491,7 @@ def connected_watts_strogatz_graph(n, k, p, tries=100, seed=None):
 
 
 @py_random_state(2)
+@nx._dispatch(graphs=None)
 def random_regular_graph(d, n, seed=None):
     r"""Returns a random $d$-regular graph on $n$ nodes.
 
@@ -614,6 +622,7 @@ def _random_subset(seq, m, rng):
 
 
 @py_random_state(2)
+@nx._dispatch(graphs=None)
 def barabasi_albert_graph(n, m, seed=None, initial_graph=None):
     """Returns a random graph using Barabási–Albert preferential attachment
 
@@ -686,6 +695,7 @@ def barabasi_albert_graph(n, m, seed=None, initial_graph=None):
 
 
 @py_random_state(4)
+@nx._dispatch(graphs=None)
 def dual_barabasi_albert_graph(n, m1, m2, p, seed=None, initial_graph=None):
     """Returns a random graph using dual Barabási–Albert preferential attachment
 
@@ -785,6 +795,7 @@ def dual_barabasi_albert_graph(n, m1, m2, p, seed=None, initial_graph=None):
 
 
 @py_random_state(4)
+@nx._dispatch(graphs=None)
 def extended_barabasi_albert_graph(n, m, p, q, seed=None):
     """Returns an extended Barabási–Albert model graph.
 
@@ -863,10 +874,10 @@ def extended_barabasi_albert_graph(n, m, p, q, seed=None):
         # Adding m new edges, if there is room to add them
         if a_probability < p and G.size() <= clique_size - m:
             # Select the nodes where an edge can be added
-            elligible_nodes = [nd for nd, deg in G.degree() if deg < clique_degree]
+            eligible_nodes = [nd for nd, deg in G.degree() if deg < clique_degree]
             for i in range(m):
-                # Choosing a random source node from elligible_nodes
-                src_node = seed.choice(elligible_nodes)
+                # Choosing a random source node from eligible_nodes
+                src_node = seed.choice(eligible_nodes)
 
                 # Picking a possible node that is not 'src_node' or
                 # neighbor with 'src_node', with preferential attachment
@@ -883,29 +894,26 @@ def extended_barabasi_albert_graph(n, m, p, q, seed=None):
                 attachment_preference.append(src_node)
                 attachment_preference.append(dest_node)
 
-                # Adjusting the elligible nodes. Degree may be saturated.
+                # Adjusting the eligible nodes. Degree may be saturated.
                 if G.degree(src_node) == clique_degree:
-                    elligible_nodes.remove(src_node)
-                if (
-                    G.degree(dest_node) == clique_degree
-                    and dest_node in elligible_nodes
-                ):
-                    elligible_nodes.remove(dest_node)
+                    eligible_nodes.remove(src_node)
+                if G.degree(dest_node) == clique_degree and dest_node in eligible_nodes:
+                    eligible_nodes.remove(dest_node)
 
         # Rewiring m edges, if there are enough edges
         elif p <= a_probability < (p + q) and m <= G.size() < clique_size:
             # Selecting nodes that have at least 1 edge but that are not
             # fully connected to ALL other nodes (center of star).
             # These nodes are the pivot nodes of the edges to rewire
-            elligible_nodes = [nd for nd, deg in G.degree() if 0 < deg < clique_degree]
+            eligible_nodes = [nd for nd, deg in G.degree() if 0 < deg < clique_degree]
             for i in range(m):
                 # Choosing a random source node
-                node = seed.choice(elligible_nodes)
+                node = seed.choice(eligible_nodes)
 
                 # The available nodes do have a neighbor at least.
                 neighbor_nodes = list(G[node])
 
-                # Choosing the other end that will get dettached
+                # Choosing the other end that will get detached
                 src_node = seed.choice(neighbor_nodes)
 
                 # Picking a target node that is not 'node' or
@@ -922,16 +930,16 @@ def extended_barabasi_albert_graph(n, m, p, q, seed=None):
                 attachment_preference.remove(src_node)
                 attachment_preference.append(dest_node)
 
-                # Adjusting the elligible nodes.
+                # Adjusting the eligible nodes.
                 # nodes may be saturated or isolated.
-                if G.degree(src_node) == 0 and src_node in elligible_nodes:
-                    elligible_nodes.remove(src_node)
-                if dest_node in elligible_nodes:
+                if G.degree(src_node) == 0 and src_node in eligible_nodes:
+                    eligible_nodes.remove(src_node)
+                if dest_node in eligible_nodes:
                     if G.degree(dest_node) == clique_degree:
-                        elligible_nodes.remove(dest_node)
+                        eligible_nodes.remove(dest_node)
                 else:
                     if G.degree(dest_node) == 1:
-                        elligible_nodes.append(dest_node)
+                        eligible_nodes.append(dest_node)
 
         # Adding new node with m edges
         else:
@@ -948,6 +956,7 @@ def extended_barabasi_albert_graph(n, m, p, q, seed=None):
 
 
 @py_random_state(3)
+@nx._dispatch(graphs=None)
 def powerlaw_cluster_graph(n, m, p, seed=None):
     """Holme and Kim algorithm for growing graphs with powerlaw
     degree distribution and approximate average clustering.
@@ -1037,6 +1046,7 @@ def powerlaw_cluster_graph(n, m, p, seed=None):
 
 
 @py_random_state(3)
+@nx._dispatch(graphs=None)
 def random_lobster(n, p1, p2, seed=None):
     """Returns a random lobster graph.
 
@@ -1087,6 +1097,7 @@ def random_lobster(n, p1, p2, seed=None):
 
 
 @py_random_state(1)
+@nx._dispatch(graphs=None)
 def random_shell_graph(constructor, seed=None):
     """Returns a random shell graph for the constructor given.
 
@@ -1144,6 +1155,7 @@ def random_shell_graph(constructor, seed=None):
 
 
 @py_random_state(2)
+@nx._dispatch(graphs=None)
 def random_powerlaw_tree(n, gamma=3, seed=None, tries=100):
     """Returns a tree with a power law degree distribution.
 
@@ -1180,6 +1192,7 @@ def random_powerlaw_tree(n, gamma=3, seed=None, tries=100):
 
 
 @py_random_state(2)
+@nx._dispatch(graphs=None)
 def random_powerlaw_tree_sequence(n, gamma=3, seed=None, tries=100):
     """Returns a degree sequence for a tree with a power law distribution.
 
@@ -1236,6 +1249,7 @@ def random_powerlaw_tree_sequence(n, gamma=3, seed=None, tries=100):
 
 
 @py_random_state(3)
+@nx._dispatch(graphs=None)
 def random_kernel_graph(n, kernel_integral, kernel_root=None, seed=None):
     r"""Returns an random graph based on the specified kernel.
 
@@ -1297,7 +1311,6 @@ def random_kernel_graph(n, kernel_integral, kernel_root=None, seed=None):
     """
     if kernel_root is None:
         import scipy as sp
-        import scipy.optimize  # call as sp.optimize
 
         def kernel_root(y, a, r):
             def my_function(b):
