@@ -1,23 +1,16 @@
-# -*- encoding: utf-8 -*-
-# test_duplication.py - unit tests for the generators.duplication module
-#
-# Copyright 2010-2019 NetworkX developers.
-#
-# This file is part of NetworkX.
-#
-# NetworkX is distributed under a BSD license; see LICENSE.txt for more
-# information.
 """Unit tests for the :mod:`networkx.generators.duplication` module.
 
 """
 import pytest
 
 from networkx.exception import NetworkXError
-from networkx.generators.duplication import duplication_divergence_graph
-from networkx.generators.duplication import partial_duplication_graph
+from networkx.generators.duplication import (
+    duplication_divergence_graph,
+    partial_duplication_graph,
+)
 
 
-class TestDuplicationDivergenceGraph(object):
+class TestDuplicationDivergenceGraph:
     """Unit tests for the
     :func:`networkx.generators.duplication.duplication_divergence_graph`
     function.
@@ -39,7 +32,7 @@ class TestDuplicationDivergenceGraph(object):
             duplication_divergence_graph(3, -1)
 
 
-class TestPartialDuplicationGraph(object):
+class TestPartialDuplicationGraph:
     """Unit tests for the
     :func:`networkx.generators.duplication.partial_duplication_graph`
     function.
@@ -71,7 +64,6 @@ class TestPartialDuplicationGraph(object):
             p = 0.5
             q = 0.5
             G = partial_duplication_graph(N, n, p, q)
-            assert len(G) == n
 
     def test_invalid_probabilities(self):
         N = 1
