@@ -1,15 +1,13 @@
-from itertools import chain
-
 import networkx as nx
 
 __all__ = ["adjacency_data", "adjacency_graph"]
 
-_attrs = dict(id="id", key="key")
+_attrs = {"id": "id", "key": "key"}
 
 
 def adjacency_data(G, attrs=_attrs):
     """Returns data in adjacency format that is suitable for JSON serialization
-    and use in Javascript documents.
+    and use in JavaScript documents.
 
     Parameters
     ----------
@@ -70,19 +68,20 @@ def adjacency_data(G, attrs=_attrs):
     data["nodes"] = []
     data["adjacency"] = []
     for n, nbrdict in G.adjacency():
-        data["nodes"].append(dict(chain(G.nodes[n].items(), [(id_, n)])))
+        data["nodes"].append({**G.nodes[n], id_: n})
         adj = []
         if multigraph:
             for nbr, keys in nbrdict.items():
                 for k, d in keys.items():
-                    adj.append(dict(chain(d.items(), [(id_, nbr), (key, k)])))
+                    adj.append({**d, id_: nbr, key: k})
         else:
             for nbr, d in nbrdict.items():
-                adj.append(dict(chain(d.items(), [(id_, nbr)])))
+                adj.append({**d, id_: nbr})
         data["adjacency"].append(adj)
     return data
 
 
+@nx._dispatch(graphs=None)
 def adjacency_graph(data, directed=False, multigraph=True, attrs=_attrs):
     """Returns graph from adjacency data format.
 
@@ -149,9 +148,9 @@ def adjacency_graph(data, directed=False, multigraph=True, attrs=_attrs):
             target = target_data.pop(id_)
             if not multigraph:
                 graph.add_edge(source, target)
-                graph[source][target].update(tdata)
+                graph[source][target].update(target_data)
             else:
                 ky = target_data.pop(key, None)
                 graph.add_edge(source, target, key=ky)
-                graph[source][target][ky].update(tdata)
+                graph[source][target][ky].update(target_data)
     return graph
