@@ -25,7 +25,7 @@ from collections import defaultdict
 
 import matplotlib.pyplot as plt
 import networkx as nx
-import numpy
+import numpy as np
 from scipy.cluster import hierarchy
 from scipy.spatial import distance
 
@@ -33,14 +33,14 @@ from scipy.spatial import distance
 def create_hc(G):
     """Creates hierarchical cluster of graph G from distance matrix"""
     path_length = nx.all_pairs_shortest_path_length(G)
-    distances = numpy.zeros((len(G), len(G)))
+    distances = np.zeros((len(G), len(G)))
     for u, p in path_length:
         for v, d in p.items():
             distances[u][v] = d
     # Create hierarchical cluster
     Y = distance.squareform(distances)
     Z = hierarchy.complete(Y)  # Creates HC using farthest point linkage
-    # This partition selection is arbitrary, for illustrive purposes
+    # This partition selection is arbitrary, for illustrative purposes
     membership = list(hierarchy.fcluster(Z, t=1.15))
     # Create collection of lists for blockmodel
     partition = defaultdict(list)
@@ -55,13 +55,13 @@ G = nx.read_edgelist("hartford_drug.edgelist")
 H = G.subgraph(next(nx.connected_components(G)))
 # Makes life easier to have consecutively labeled integer nodes
 H = nx.convert_node_labels_to_integers(H)
-# Create parititions with hierarchical clustering
+# Create partitions with hierarchical clustering
 partitions = create_hc(H)
 # Build blockmodel graph
 BM = nx.quotient_graph(H, partitions, relabel=True)
 
 # Draw original graph
-pos = nx.spring_layout(H, iterations=100)
+pos = nx.spring_layout(H, iterations=100, seed=83)  # Seed for reproducibility
 plt.subplot(211)
 nx.draw(H, pos, with_labels=False, node_size=10)
 
@@ -71,7 +71,7 @@ edge_width = [(2 * d["weight"]) for (u, v, d) in BM.edges(data=True)]
 # Set positions to mean of positions of internal nodes from original graph
 posBM = {}
 for n in BM:
-    xy = numpy.array([pos[u] for u in BM.nodes[n]["graph"]])
+    xy = np.array([pos[u] for u in BM.nodes[n]["graph"]])
     posBM[n] = xy.mean(axis=0)
 plt.subplot(212)
 nx.draw(BM, posBM, node_size=node_size, width=edge_width, with_labels=False)
